/** Daca probabilitatea ca un eveniment sa aiba loc este P, atunci
 * Numarul Asteptat de Incercari (Expected Number of Trials)
   inainte ca evenimentul sa aiba loc este:
 * T = 1 / P
 *
 * P = Nf / Nt
 *
 * Nf = nr de cazuri favorabile
 * Nt = nr total de cazuri
 */
#include <iostream>
#include <string>
#include <algorithm>
using namespace std;

class SeatingPlan {
private:
	using ll = long long;

	ll Cmmdc(ll a, ll b)
	{
		return b ? Cmmdc(b, a % b) : a;
	}

	long long C[128][128];       // Combinari
	bool ok[256];                // ok[mask] = true daca nu sunt doi biti 1 adiacenti in masca
	int count[256];              // count[mask] - nr bitilor 1 in masca mask (numarul de cheaters)
	long long nr[128][128][32];  // nr[c][j][q] = nr de posibilitati de a plasa q de 1 intr-o matrice m x c daca ultima coloana e codificata in masca j

public:
	string expectedTrial(int m, int n, int k)
	{
		if (m > n) 	swap(m, n);

		for (int i = 1; i <= m * n; i++)
			C[i][0] = C[i][i] = 1;
		for (int i = 2; i <= m * n; i++)
			for (int j = 1; j < i; j++)
				C[i][j] = C[i - 1][j - 1] + C[i - 1][j];

		count[0] = 0;
		ok[0] = true;
		for (int mask = 1, j; mask < 1 << m; ++mask)
		{
			count[mask] = count[mask & (mask - 1)] + 1;      // nr de biti 1 din fiecare masca

			// determin daca in masca mask exista sau nu secventa 11 consecutiva
			j = 0;
			while (j < m && ((3 << j) & mask) != 3 << j)    // deplasez la stanga masca "11" cu j = 0, 1, ...j - 1 pozitii
				j++;
			ok[mask] = (j == m);
		}

		nr[0][0][0] = 1;  // o posibilitate sa plasez pe prima coloana (0) 0 biti si sa am in total 0 biti
		for (int c = 1; c <= n; c++)
			for (int i = 0; i < 1 << m; i++)          // coloana c-1  in masca i
				for (int j = 0; j < 1 << m; j++)      // coloana c in masca j
					if (ok[i] && ok[j] && ((i & j) == 0))  // i sau j nu au 2 biti 1 alaturati si intersectia lor e vida (nu au biti 1 pe aceeasi pozitie)
						for (int q = 0; q + count[j] <= k; q++)     // q - nr total de biti pana la coloana c - 1
							nr[c][j][q + count[j]] += nr[c - 1][i][q];

		long long a = 0, b;
		for (int i = 0; i < 1 << m; ++i)     // nr de posibilitati ca pe coloana n sa am i = 0, 1, ... m biti de 1
			a += nr[n][i][k];                // si totalul de biti (cheaters) pana la col n sa fie k


		if (a == 0)
			return "Impossible!";

		// a - n de cazuri favorabile
	    // b - nr total de cazuri
		b = C[m * n][k];
		long long cmd = Cmmdc(a, b);
		a /= cmd;
		b /= cmd;
		return std::to_string(b) + string("/") + std::to_string(a);
	}
};

SeatingPlan s;
int main()
{

	cout << s.expectedTrial(1, 79, 17);
}
